@@ -32,24 +32,16 @@
 #include "file.hpp"
 #include "lib.h"
 
+struct SearchResult {
+	const gchar *bookname;
+	const gchar *def;
+	const gchar *exp;
+};
+
 using std::string;
 using std::vector;
 class MStarDict;
 
-//this structure is wrapper and it need for unification
-//results of search whith return Dicts class
-struct TSearchResult {
-    string bookname;
-    string def;
-    string exp;
-
-    TSearchResult(const string &bookname_, const string &def_, const string &exp_)
-    :bookname(bookname_), def(def_), exp(exp_) {
-}};
-
-typedef vector < TSearchResult > TSearchResultList;
-typedef TSearchResultList::iterator PSearchResult;
-
 //this class is wrapper around Dicts class for easy use
 //of it
 class Library:public Libs {
@@ -68,7 +60,9 @@ class Library:public Libs {
 			 const char *sWord,
 			 CurrentIndex *iIndex,
 			 int iLib,
-			 TSearchResultList &res_list);
+			 GList **result_data);
+    void FreeResultData(GList *result_data);
+
     bool SimpleLookup(const gchar *sWord,
 		      CurrentIndex *piIndex);
     bool LookupWithFuzzy(const gchar *sWord);
